#! /bin/sh
#
#  yacc to dot (yacc2dot) - convert yacc files to dot graph descriptions
#
#  Author: Philippe Oechslin (oechslin@di.epfl.ch)
#  version: 1.0 date: 3.23.95
#
#  This gawk programm produces a graph layout for the Dot program from a 
#  yacc grammar. 
#
#  standard usage:   yacc2dot < test.y | dot -Tps > test.ps
#
#  Currently it has following restrictions:
#  - exactly one line per rule
#  - :, | and ; must be surrounded by whitespaces,
#  - ; has to stand on a separate line (standard style)
#  - c-type comments may not extend over multiple lines
#  - tokens must be uppercase, non-tokens must contain lowercase letters
#
# This is a short example of a grammar that can be parsed:
# %%
# ovarlst	: /* empty */
# 		| varlst
#		;
#
# varlst        : ID {use($1);}
#               | varlst ID {use($2);}
#               ;
# %%

# The resctrictions could be eliminated by using other field separators 
# and by parsing the token declarations. 
#
# Any comments and suggestions are welcome.

gawk ' #gnu awk, may not work with other awks
BEGIN {

  print "digraph g {"

# page size settings for A4 paper:

# use this for multipage output:
#  printf "graph [page = \"8.2,11.6\", size = \"14.4,10.6\" , center = on,  "

# use this for single page output:
  printf " graph [size = \"7.5,11\", "

  print "center = on, ratio = fill];"
  print "node[shape = record];"

  section = 0;

}

/%%/ {section ++}

{ if (section != 1) next;} # only parse the lines between the first and
                           # second %%

/:/  { # handle left hand side (lhs) of rule

# get rid of actions
  gsub(/{[^}]*}/,"");
# get rid of comments
  gsub(/\/\*([^\*]*\*)+\//,"")

  if (!($1 in nodelist)) {
    nodelist[$1] = 0
    print "\nsubgraph cluster_" $1 " {"
    print $1 ";\n}"
    }
  lhs = $1
  }

/[\|:]/     { # handle rhs of rule

  print "\nsubgraph cluster_" lhs " {"  

# get rid of actions
  gsub(/{[^}]*}/,"");
# get rid of comments
  gsub(/\/\*([^\*]*\*)+\//,"")

# get rid of lhs if first line of a rule
  if ($2 == ":") gsub(/^[^:]*/,"");
#  print "\n" $0

# print node with one field for each word on the rhs
  printf lhs nodelist[lhs] " [label = \"";

  if (NF > 1) {
    for (i = 2; i<NF; i++)
      printf "<" $i ">" $i "|";
    printf "<" $(NF) ">" $(NF) "\"];\n"
  }
  else print " empty \"];"

# link this node to the lhs
  print lhs " -> " lhs nodelist[lhs] ";"

# close cluster
  print "}"

# link the field which are not tokens to the corresponding rule
  for (i = 2; i<=NF; i++)
    if (toupper($i) != $i)
      print lhs nodelist[lhs] ":" $i " -> " $i ";";

  nodelist[lhs]++

}

END {
 print "}";
}'




